/*************************************************
* Win32 CAPI EntropySource Source File           *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/es_capi.h>
#include <botan/conf.h>
#include <windows.h>
#include <wincrypt.h>

namespace Botan {

namespace {

/*************************************************
* CSP Handle                                     *
*************************************************/
class CSP_Handle
   {
   public:
      CSP_Handle(u64bit);
      ~CSP_Handle();

      HCRYPTPROV get_handle() const { return handle; }
   private:
      HCRYPTPROV handle;
   };

/*************************************************
* Initialize a CSP Handle                        *
*************************************************/
CSP_Handle::CSP_Handle(u64bit capi_provider)
   {
   DWORD prov_type = (DWORD)capi_provider;

   if(CryptAcquireContext(&handle, 0, 0, prov_type, CRYPT_VERIFYCONTEXT))
      return;
   if(CryptAcquireContext(&handle, 0, 0, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT))
      return;
   throw Exception("Win32_CAPI_EntropySource: CryptAcquireContext failed");
   }

/*************************************************
* Destroy a CSP Handle                           *
*************************************************/
CSP_Handle::~CSP_Handle()
   {
   CryptReleaseContext(handle, 0);
   }

}

/*************************************************
* Gather Entropy from Win32 CAPI                 *
*************************************************/
u32bit Win32_CAPI_EntropySource::slow_poll(byte output[], u32bit length)
   {
   CSP_Handle csp(prov_type);
   if(!CryptGenRandom(csp.get_handle(), length, output))
      return 0;
   return length;
   }

/*************************************************
* Gather Entropy from Win32 CAPI                 *
*************************************************/
Win32_CAPI_EntropySource::Win32_CAPI_EntropySource(const std::string& prov)
   {
   std::string capi_prov = prov;
   if(capi_prov == "")
      capi_prov = Config::get_string("rng/ms_capi_prov_type");

   prov_type = PROV_RSA_FULL;
   if(capi_prov == "RSA_FULL")  prov_type = PROV_RSA_FULL;
   if(capi_prov == "INTEL_SEC") prov_type = PROV_INTEL_SEC;
   if(capi_prov == "FORTEZZA")  prov_type = PROV_FORTEZZA;
   if(capi_prov == "RNG")       prov_type = PROV_RNG;
   }

}
